@@ -22,8 +22,6 @@ only statement.
 
 """
 
-from nuitka import Options
-
 from .CodeHelpers import generateExpressionCode
 from .ErrorCodes import getReleaseCode
 
@@ -37,20 +35,11 @@ def generateExpressionOnlyCode(statement, emit, context):
 
 
 def getStatementOnlyCode(value, emit, context):
-    # TODO: Introduce "void" as a C type, which discards all assignments
-    # as a no-op.
-    if Options.isExperimental("enable_void_ctype"):
-        tmp_name = context.allocateTempName(
-            base_name = "unused",
-            type_name = "void",
-            unique    = True
-        )
-    else:
-        tmp_name = context.allocateTempName(
-            base_name = "unused",
-            type_name = "NUITKA_MAY_BE_UNUSED PyObject *",
-            unique    = True
-        )
+    tmp_name = context.allocateTempName(
+        base_name = "unused",
+        type_name = "void",
+        unique    = True
+    )
 
     generateExpressionCode(
         expression = value,
