@@ -22,7 +22,13 @@ This also includes writing back to locals for exec statements.
 
 from nuitka.nodes.shapes.BuiltinTypeShapes import ShapeTypeDict
 
+from .CodeHelpers import (
+    decideConversionCheckNeeded,
+    withObjectCodeTemporaryAssignment
+)
+from .Emission import SourceCodeCollector
 from .ErrorCodes import getErrorExitBoolCode
+from .Indentation import indented
 from .PythonAPICodes import generateCAPIObjectCode
 from .templates.CodeTemplatesVariables import (
     template_set_locals_dict_value,
@@ -30,19 +36,27 @@ from .templates.CodeTemplatesVariables import (
     template_update_locals_dict_value,
     template_update_locals_mapping_value
 )
-from .VariableCodes import getLocalVariableDeclaration
+from .VariableCodes import (
+    getLocalVariableDeclaration,
+    getVariableReferenceCode
+)
+from .VariableDeclarations import VariableDeclaration
 
 
 def generateBuiltinLocalsRefCode(to_name, expression, emit, context):
     locals_scope = expression.getLocalsScope()
 
     locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
-    emit(
-        "%s = %s;" % (
-            to_name,
-            locals_declaration,
+
+    with withObjectCodeTemporaryAssignment(to_name, "locals_ref_value", expression, emit, context) \
+      as value_name:
+
+        emit(
+            "%s = %s;" % (
+                value_name,
+                locals_declaration,
+            )
         )
-    )
 
 
 def generateBuiltinLocalsCode(to_name, expression, emit, context):
@@ -69,68 +83,62 @@ def generateBuiltinLocalsCode(to_name, expression, emit, context):
     # about local variables at all.
     assert not provider.isCompiledPythonModule(), provider
 
-    if updated:
-        locals_scope = expression.getLocalsScope()
+    with withObjectCodeTemporaryAssignment(to_name, "locals_ref_value", expression, emit, context) \
+      as value_name:
 
-        locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
-        is_dict = locals_scope.getTypeShape() is ShapeTypeDict
-        # For Python3 it may really not be a dictionary.
+        if updated:
+            locals_scope = expression.getLocalsScope()
 
-        # TODO: Creation is not needed for classes.
-        emit(
-            """\
+            locals_declaration = context.addLocalsDictName(locals_scope.getCodeName())
+            is_dict = locals_scope.getTypeShape() is ShapeTypeDict
+            # For Python3 it may really not be a dictionary.
+
+            # TODO: Creation is not needed for classes.
+            emit(
+                """\
 if (%(locals_dict)s == NULL) %(locals_dict)s = PyDict_New();
 %(to_name)s = %(locals_dict)s;
 Py_INCREF( %(to_name)s );""" % {
-                "to_name"     : to_name ,
-                "locals_dict" : locals_declaration,
-            }
-        )
-        context.addCleanupTempName(to_name)
-
-        initial = False
-    else:
-        emit(
-            "%s = PyDict_New();" % (
-                to_name,
+                    "to_name"     : value_name,
+                    "locals_dict" : locals_declaration,
+                }
             )
-        )
+            context.addCleanupTempName(value_name)
 
-        context.addCleanupTempName(to_name)
+            initial = False
+        else:
+            emit(
+                "%s = PyDict_New();" % (
+                    to_name,
+                )
+            )
 
-        initial = True
-        is_dict = True
+            context.addCleanupTempName(value_name)
 
-    for local_var, variable_trace in _sorted(variable_traces):
-        _getVariableDictUpdateCode(
-            target_name    = to_name,
-            variable       = local_var,
-            variable_trace = variable_trace,
-            is_dict        = is_dict,
-            initial        = initial,
-            emit           = emit,
-            context        = context
-        )
+            initial = True
+            is_dict = True
+
+        for local_var, variable_trace in _sorted(variable_traces):
+            _getVariableDictUpdateCode(
+                target_name    = value_name,
+                variable       = local_var,
+                variable_trace = variable_trace,
+                is_dict        = is_dict,
+                initial        = initial,
+                emit           = emit,
+                context        = context
+            )
 
 
 def generateBuiltinGlobalsCode(to_name, expression, emit, context):
-    # Functions used for generation all accept expression, but this one does
-    # not use it. pylint: disable=unused-argument
-
-    getLoadGlobalsCode(
-        to_name = to_name,
-        emit    = emit,
-        context = context
-    )
-
-
-def getLoadGlobalsCode(to_name, emit, context):
-    emit(
-        "%(to_name)s = (PyObject *)moduledict_%(module_identifier)s;" % {
-            "to_name"           : to_name,
-            "module_identifier" : context.getModuleCodeName()
-        },
-    )
+    with withObjectCodeTemporaryAssignment(to_name, "globals_value", expression, emit, context) \
+      as value_name:
+        emit(
+            "%(to_name)s = (PyObject *)moduledict_%(module_identifier)s;" % {
+                "to_name"           : value_name,
+                "module_identifier" : context.getModuleCodeName()
+            },
+        )
 
 
 def _getLocalVariableList(provider):
@@ -159,8 +167,19 @@ def _getVariableDictUpdateCode(target_name, variable, variable_trace, initial,
 
     variable_c_type = variable_declaration.getCType()
 
-    test_code = variable_c_type.getLocalVariableInitTestCode(variable_declaration)
-    access_code = variable_c_type.getLocalVariableObjectAccessCode(variable_declaration)
+    test_code = variable_c_type.getLocalVariableInitTestCode(variable_declaration, False)
+
+    access_code = SourceCodeCollector()
+
+    getVariableReferenceCode(
+        to_name          = VariableDeclaration("PyObject *", "value", None, None),
+        variable         = variable,
+        variable_trace   = variable_trace,
+        needs_check      = False,
+        conversion_check = True,
+        emit             = access_code,
+        context          = context
+    )
 
     if is_dict:
         if initial:
@@ -175,7 +194,7 @@ def _getVariableDictUpdateCode(target_name, variable, variable_trace, initial,
                     constant = variable.getName()
                 ),
                 "test_code"   : test_code,
-                "access_code" : access_code
+                "access_code" : indented(access_code.codes)
             }
         )
     else:
@@ -207,27 +226,29 @@ def _getVariableDictUpdateCode(target_name, variable, variable_trace, initial,
 
 def generateBuiltinDir1Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
-        to_name    = to_name,
-        capi       = "PyObject_Dir",
-        arg_desc   = (
+        to_name          = to_name,
+        capi             = "PyObject_Dir",
+        arg_desc         = (
             ("dir_arg", expression.getValue()),
         ),
-        may_raise  = expression.mayRaiseException(BaseException),
-        source_ref = expression.getCompatibleSourceReference(),
-        emit       = emit,
-        context    = context
+        may_raise        = expression.mayRaiseException(BaseException),
+        conversion_check = decideConversionCheckNeeded(to_name, expression),
+        source_ref       = expression.getCompatibleSourceReference(),
+        emit             = emit,
+        context          = context
     )
 
 
 def generateBuiltinVarsCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
-        to_name    = to_name,
-        capi       = "LOOKUP_VARS",
-        arg_desc   = (
+        to_name          = to_name,
+        capi             = "LOOKUP_VARS",
+        arg_desc         = (
             ("vars_arg", expression.getSource()),
         ),
-        may_raise  = expression.mayRaiseException(BaseException),
-        source_ref = expression.getCompatibleSourceReference(),
-        emit       = emit,
-        context    = context
+        may_raise        = expression.mayRaiseException(BaseException),
+        conversion_check = decideConversionCheckNeeded(to_name, expression),
+        source_ref       = expression.getCompatibleSourceReference(),
+        emit             = emit,
+        context          = context
     )
