@@ -45,7 +45,7 @@ def matchesModuleNameToPatterns(module_name, patterns):
     for pattern in patterns:
         if module_name == pattern:
             return True, "is exact match of %r" % pattern
-        elif module_name.startswith(pattern):
+        elif module_name.startswith(pattern + '.'):
             return True, "is package content of %r" % pattern
         elif fnmatch.fnmatch(module_name, pattern):
             return True, "matches pattern %r" % pattern
@@ -68,94 +68,82 @@ def _recurseTo(module_package, module_filename, module_relpath, module_kind,
         is_shlib = module_kind == "shlib"
     )
 
-    # Check if the module name is known. In order to avoid duplicates,
-    # learn the new filename, and continue build if its not.
-    if not ImportCache.isImportedModuleByName(module.getFullName()):
-        logRecursion(
-            "Recurse to import '%s' from '%s'. (%s)",
-            module.getFullName(),
-            module_relpath,
-            reason
-        )
+    logRecursion(
+        "Recurse to import '%s' from '%s'. (%s)",
+        module.getFullName(),
+        module_relpath,
+        reason
+    )
 
-        if module_kind == "py" and source_filename is not None:
-            try:
-                source_code = readSourceCodeFromFilename(
-                    module_name     = module.getFullName(),
-                    source_filename = source_filename
-                )
+    if module_kind == "py" and source_filename is not None:
+        try:
+            source_code = readSourceCodeFromFilename(
+                module_name     = module.getFullName(),
+                source_filename = source_filename
+            )
 
-                Building.createModuleTree(
-                    module      = module,
-                    source_ref  = source_ref,
-                    source_code = source_code,
-                    is_main     = False
-                )
-            except (SyntaxError, IndentationError) as e:
-                if module_filename not in Importing.warned_about:
-                    Importing.warned_about.add(module_filename)
+            Building.createModuleTree(
+                module      = module,
+                source_ref  = source_ref,
+                source_code = source_code,
+                is_main     = False
+            )
+        except (SyntaxError, IndentationError) as e:
+            if module_filename not in Importing.warned_about:
+                Importing.warned_about.add(module_filename)
 
-                    warning(
-                        """\
+                warning(
+                    """\
 Cannot recurse to import module '%s' (%s) because of '%s'""",
-                        module_relpath,
-                        module_filename,
-                        e.__class__.__name__
-                    )
+                    module_relpath,
+                    module_filename,
+                    e.__class__.__name__
+                )
 
-                return None, False
-            except Building.CodeTooComplexCode:
-                if module_filename not in Importing.warned_about:
-                    Importing.warned_about.add(module_filename)
+            return None, False
+        except Building.CodeTooComplexCode:
+            if module_filename not in Importing.warned_about:
+                Importing.warned_about.add(module_filename)
 
-                    warning(
-                        """\
+                warning(
+                    """\
 Cannot recurse to import module '%s' (%s) because code is too complex.""",
-                        module_relpath,
-                        module_filename,
+                    module_relpath,
+                    module_filename,
+                )
+
+                if Options.isStandaloneMode():
+                    module = makeUncompiledPythonModule(
+                        module_name   = module.getFullName(),
+                        filename      = module_filename,
+                        bytecode      = marshal.dumps(
+                            compile(
+                                source_code,
+                                module_filename,
+                                "exec",
+                                dont_inherit = True
+                            )
+                        ),
+                        is_package    = module.isCompiledPythonPackage(),
+                        user_provided = True,
+                        technical     = False
                     )
 
-                    if Options.isStandaloneMode():
-                        module = makeUncompiledPythonModule(
-                            module_name   = module.getFullName(),
-                            filename      = module_filename,
-                            bytecode      = marshal.dumps(
-                                compile(
-                                    source_code,
-                                    module_filename,
-                                    "exec",
-                                    dont_inherit = True
-                                )
-                            ),
-                            is_package    = module.isCompiledPythonPackage(),
-                            user_provided = True,
-                            technical     = False
-                        )
+                    ModuleRegistry.addUncompiledModule(module)
 
-                        ModuleRegistry.addUncompiledModule(module)
+            return None, False
 
-                return None, False
+    ImportCache.addImportedModule(module)
 
-        ImportCache.addImportedModule(module)
-
-        is_added = True
-    else:
-        module = ImportCache.getImportedModuleByName(
-            module.getFullName()
-        )
-
-        is_added = False
-
-    return module, is_added
+    return module, True
 
 
 def recurseTo(module_package, module_filename, module_relpath, module_kind,
               reason):
     if ImportCache.isImportedModuleByPath(module_relpath):
-        module = ImportCache.getImportedModuleByPath(module_relpath)
-
-        if module.getCompileTimeFilename().endswith("__init__.py") and \
-           module.getPackage() != module_package:
+        try:
+            module = ImportCache.getImportedModuleByPath(module_relpath, module_package)
+        except KeyError:
             module = None
     else:
         module = None
