@@ -80,6 +80,14 @@ class ExpressionBuiltinFormat(ExpressionChildrenHavingBase):
 Removed useless 'format' on '%s' value.""" % value.getTypeShape().getTypeName()
                 )
 
+        # TODO: Provide "__format__" slot based handling.
+
+        # Any code could be run, note that.
+        trace_collection.onControlFlowEscape(self)
+
+        # Any exception may be raised.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
 
     getValue = ExpressionChildrenHavingBase.childGetter("value")
@@ -147,6 +155,8 @@ class ExpressionBuiltinId(ExpressionBuiltinSingleArgBase):
             node       = self
         )
 
+        del self.parent
+
         return result, "new_statements", """\
 Removed id taking for unused result."""
 
