@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,7 +29,7 @@ import runpy
 import sys
 import tempfile
 
-import vmprof  # @UnresolvedImport pylint: disable=F0401,I0021
+import vmprof  # @UnresolvedImport pylint: disable=I0021,import-error
 
 
 def _namelen(e):
